import numpy as np

from cs598apk import factorial


class Translation:
    def __init__(self, exp):
        self.exp = exp

    def multipole_to_multipole(self, M, cold, cnew):
        p = self.exp.indices
        d = cnew - cold

        N = np.zeros(M.shape)
        for i in range(self.exp.nindex):
            for j in range(self.exp.nindex):
                if not np.all(p[j] <= p[i]):
                    continue

                N[i] += M[j] * np.product(d ** (p[i] - p[j])) / \
                    factorial(p[i] - p[j])

        return N

    def multipole_to_local(self, M, cold, cnew):
        p = self.exp.indices

        N = np.zeros(M.shape)
        for i in range(self.exp.nindex):
            for j in range(self.exp.nindex):
                if not np.all(p[j] <= p[i]):
                    continue

                N[i] += M[j] * self.exp.derivatives[i, j](*cnew, *cold) / \
                    factorial(p[i] - p[j])

        return N

    def local_to_local(self, L, cold, cnew):
        p = self.exp.indices
        d = cnew - cold

        N = np.zeros(L.shape)
        for i in range(self.exp.nindex):
            for j in range(self.exp.nindex):
                if not np.all(p[j] >= p[i]):
                    continue

                N[i] += L[j] * np.product(d ** (p[j] - p[i])) / \
                    factorial(p[i] - p[j])

        return N
