import numpy as np
import matplotlib.pyplot as mp

import time
import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def dcurve_dt(t):
    dx = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * np.cos(2.0 * t) - \
        (3.0 / 4.0) * np.sin(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    dy = np.cos(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0) + \
        np.sin(t - np.pi / 4.0) * np.cos(4.0 * t) * 2.0

    return np.array([dx, dy])


def test_eigenvalues(npanels, limit, filename):
    # number of quadrature nodes
    default_q = 11
    # expansion order
    default_p = 3

    for p in [3, 5, 7]:
        # define geometry
        geometry = cs.ParametricGeometry(curve, dcurve_dt)
        # define solution representation
        K = cs.DoubleLayerKernel(cs.GreenLaplace())
        K = cs.ExpandedKernel(K, order=p)
        # define quadrature
        quad = cs.QBX(geometry, npanels, order=default_q)

        # plot eigenspectrum of expansion
        t_start = time.time()
        D = quad.boundary_operator(K, limit=limit)
        t_end = time.time()
        print("p = {:02}: elapsed time: {}s".format(p, t_end - t_start))

        if limit == cs.QBXExpansion.Boundary:
            D = 0.5 * np.eye(D.shape[0]) + D

        _, eigs, _ = np.linalg.svd(D)
        mp.semilogy(eigs, label="$p = {}$".format(p))

    mp.legend()
    mp.savefig("{}_p.png".format(filename),
               bbox_inches="tight", dpi=300)
    mp.clf()

    for q in [7, 11, 15]:
        # define geometry
        geometry = cs.ParametricGeometry(curve, dcurve_dt)
        # define solution representation
        K = cs.DoubleLayerKernel(cs.GreenLaplace())
        K = cs.ExpandedKernel(K, order=default_p)
        # define quadrature
        quad = cs.QBX(geometry, npanels, order=q)

        # plot eigenspectrum of expansion
        t_start = time.time()
        D = quad.boundary_operator(K, limit=limit)
        t_end = time.time()
        print("q = {:02}: elapsed time: {}s".format(q, t_end - t_start))

        if limit == cs.QBXExpansion.Boundary:
            D = 0.5 * np.eye(D.shape[0]) + D

        _, eigs, _ = np.linalg.svd(D)
        mp.semilogy(eigs, label="$q = {}$".format(q))

    mp.legend()
    mp.savefig("{}_q.png".format(filename),
               bbox_inches="tight", dpi=300)
    mp.clf()


if __name__ == "__main__":
    # initial number of panels
    npanels = 80

    limit = cs.QBXExpansion.Interior
    test_eigenvalues(npanels, limit, "double_layer_interior")
    limit = cs.QBXExpansion.Exterior
    test_eigenvalues(npanels, limit, "double_layer_exterior")
    limit = cs.QBXExpansion.Boundary
    test_eigenvalues(npanels, limit, "double_layer_boundary")
