import numpy as np
import matplotlib.pyplot as mp

import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def dcurve_dt(t):
    dx = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * np.cos(2.0 * t) - \
        (3.0 / 4.0) * np.sin(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    dy = np.cos(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0) + \
        np.sin(t - np.pi / 4.0) * np.cos(4.0 * t) * 2.0

    return np.array([dx, dy])


def plot_panels(qbx, filename):
    for r, _ in qbx.iterranges():
        mp.plot(qbx.x[0, r], qbx.x[1, r], "o-")

    mp.xlim([-2.0, 2.0])
    mp.ylim([-2.0, 2.0])
    mp.savefig(filename, bbox_inches="tight", dpi=300)
    mp.clf()


def plot_normal_vectors(qbx, filename):
    for r, _ in qbx.iterranges():
        line = mp.plot(qbx.x[0, r], qbx.x[1, r], "o-")
        color = line[0].get_color()

        for i in r:
            x = qbx.x[:, i]
            y = 0.5 * cs.normalize(qbx.x[:, i] + qbx.n[:, i])
            mp.arrow(*x, *y, color=color)

    mp.xlim([-2.0, 2.0])
    mp.ylim([-2.0, 2.0])
    mp.savefig(filename, bbox_inches="tight", dpi=300)
    mp.clf()


def plot_centers(qbx, filename):
    for r, _ in qbx.iterranges():
        line = mp.plot(qbx.x[0, r], qbx.x[1, r])
        color = line[0].get_color()

        mp.plot(qbx.ci[0, r], qbx.ci[1, r], "o", color=color)
        mp.plot(qbx.ce[0, r], qbx.ce[1, r], "o", color=color)

    mp.xlim([-2.0, 2.0])
    mp.ylim([-2.0, 2.0])
    mp.savefig(filename, bbox_inches="tight", dpi=300)
    mp.clf()


if __name__ == "__main__":
    # number of panels
    npanels = 15
    # quadrature order
    q = 11

    # create geometry
    geometry = cs.ParametricGeometry(curve, dcurve_dt)
    # create quadrature
    qbx = cs.QBX(geometry, npanels, order=q, adapt=True)

    plot_panels(qbx, "qbx_adapt_nodes.png")
    plot_normal_vectors(qbx, "qbx_adapt_normals.png")
    plot_centers(qbx, "qbx_adapt_centers.png")
