import numpy as np
import matplotlib.pyplot as mp

import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def dcurve_dt(t):
    dx = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * np.cos(2.0 * t) - \
        (3.0 / 4.0) * np.sin(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    dy = np.cos(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0) + \
        np.sin(t - np.pi / 4.0) * np.cos(4.0 * t) * 2.0

    return np.array([dx, dy])


def plot_kernel(qbx, kernel, filename):
    # number of targets
    N = 100

    for r, i in qbx.iterranges():
        print(i)
        mp.plot(qbx.x[0, r], qbx.x[1, r], "-")
        if i % 4 != 0:
            continue

        # evaluate kernel around expansion center in the middle of the panel
        j = list(r)[len(r) // 2]

        # create target coordinates
        theta = np.linspace(0.0, 2.0 * np.pi, N).reshape(-1, 1)
        r = np.linspace(0.0, 0.9 * qbx.ratio * qbx.h[j], N).reshape(1, -1)
        x = qbx.ce[:, j:j + 1] + \
            np.array([r * np.cos(theta), r * np.sin(theta)]).reshape(2, -1)

        args = {
            "x0": x[0, :].reshape(-1, 1),
            "x1": x[1, :].reshape(-1, 1),
            "y0": qbx.x[0, :].reshape(1, -1),
            "y1": qbx.x[1, :].reshape(1, -1),
            "c0": np.full((x.shape[1], 1), qbx.ce[0, j]),
            "c1": np.full((x.shape[1], 1), qbx.ce[1, j])
        }
        K = kernel(**args)
        K = K @ qbx.w

        mp.contourf(x[0, :].reshape(N, N),
                    x[1, :].reshape(N, N),
                    K.reshape(N, N))

    mp.xlim([-2.0, 2.0])
    mp.ylim([-2.0, 2.0])
    mp.savefig(filename, bbox_inches="tight", dpi=300)
    mp.clf()


if __name__ == "__main__":
    # QBX expansion order
    p = 3
    # quadrature order
    q = 11
    # number of panels
    npanels = 15

    # create geometry
    geometry = cs.ParametricGeometry(curve, dcurve_dt)
    # create quadrature
    qbx = cs.QBX(geometry, npanels, order=q, adapt=True)

    kernel = cs.GreenLaplace()
    plot_kernel(qbx, kernel, "qbx_kernel_single_layer.png")
    kernel = cs.ExpandedKernel(kernel, order=p)
    plot_kernel(qbx, kernel, "qbx_kernel_expanded.png")
