import numpy as np
import numpy.linalg as nl
# import scipy.spatial.distance as ssd
import matplotlib.pyplot as mp

import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def dcurve_dt(t):
    dx = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * np.cos(2.0 * t) - \
        (3.0 / 4.0) * np.sin(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    dy = np.cos(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0) + \
        np.sin(t - np.pi / 4.0) * np.cos(4.0 * t) * 2.0

    return np.array([dx, dy])


def plot_proxies(qbx, nproxies, nranges, filename):
    # create blocks
    rindex = np.array_split(np.arange(0, np.prod(qbx.shape)), nranges)

    # plot proxy points for each block
    for i in range(q + 1):
        # find block center of mass
        center = np.mean(qbx.x[:, rindex[i]], axis=1).reshape(-1, 1)

        # find proxy minimum radius
        ri = np.max(nl.norm(center - qbx.ci[:, rindex[i]], axis=0) +
                    qbx.ratio * qbx.h[rindex[i]])
        re = np.max(nl.norm(center - qbx.ce[:, rindex[i]], axis=0) +
                    qbx.ratio * qbx.h[rindex[i]])
        # r = 1.2 * np.max(nl.norm(center - qbx.x[:, rindex[i]], axis=0))
        r = 1.2 * max(ri, re)

        # construct proxy points
        theta = np.linspace(0.0, 2.0 * np.pi, nproxies)
        proxies = center + \
            r * np.array([np.cos(theta), np.sin(theta)])

        # find additional nearby points
        mask = nl.norm(center - qbx.x, axis=0) <= r
        mask[rindex[i]] = False
        additional_proxies = qbx.x[:, mask]

        # mp.plot(qbx.x[0], qbx.x[1], '--', color='k')
        mp.plot(proxies[0], proxies[1],
                'o', ms=4.0, color="tab:green")
        mp.plot(qbx.x[0, rindex[i]], qbx.x[1, rindex[i]],
                'o', ms=3.0, color="tab:blue")
        mp.plot(additional_proxies[0], additional_proxies[1],
                'o', ms=3.0, color="tab:red")

        for j in rindex[i]:
            r = qbx.ratio * qbx.h[j]
            c = mp.Circle(qbx.ci[:, j], radius=r,
                          color="gray", alpha=0.1)
            mp.gca().add_patch(c)

        # mp.xlim([-2.0, 2.0])
        # mp.ylim([-2.0, 2.0])
        mp.savefig("{}_{:02}.png".format(filename, i),
                   bbox_inches="tight", dpi=300)
        mp.clf()


if __name__ == "__main__":
    # order of QBX expansion
    p = 3
    # nodes in Gauss quadrature
    q = 7
    # number of panels
    npanels = 10
    # number of proxy points
    nproxies = 42
    # number of blocks
    nranges = q + 1

    # define geometry
    geometry = cs.ParametricGeometry(curve, dcurve_dt)
    # define solution representation
    K = cs.DoubleLayerKernel(cs.GreenLaplace())
    K = cs.ExpandedKernel(K, order=p)
    # define quadrature
    qbx = cs.QBX(geometry, npanels, order=q, adapt=True)

    plot_proxies(qbx, nproxies, nranges, "qbx_skeleton")
