import numpy as np
import matplotlib.pyplot as mp

import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def plot_boxes(tree, x, filename):
    tree.plot(fill=None)

    mp.plot(x[0], x[1], "o", ms=5.0)
    mp.gcf().set_size_inches(20, 20)

    mp.savefig(filename, bbox_inches="tight", dpi=300)
    mp.clf()


def plot_interactions(tree, x, filename):
    boxes = tree.list_boxes(all_boxes=False)
    boxes = np.random.choice(boxes, 10)

    for i, box in enumerate(boxes):
        box.plot_interactions()
        tree.plot(fill=None)

        mp.gcf().set_size_inches(20, 20)
        mp.xlim(tree.bbox[0, :])
        mp.ylim(tree.bbox[1, :])
        mp.savefig("{}_{:02}.png".format(filename, i + 2),
                   bbox_inches="tight", dpi=300)
        mp.clf()


if __name__ == "__main__":
    # number of points
    N = 1024

    # points
    x = np.linspace(0.0, 2.0 * np.pi, N)
    x = curve(x) + 0.1 * np.random.random((2, N))

    # domain
    bbox = np.array([np.min(x, axis=1), np.max(x, axis=1)]).reshape(2, 2).T

    # build tree
    tree = cs.Quadtree(bbox)
    for i in range(N):
        tree.insert(i, np.array([x[:, i], x[:, i]]).reshape(2, 2).T)
    tree.build_interactions()

    plot_boxes(tree, x, "quadtree_boxes.png")
    plot_interactions(tree, x, "quadtree_list")
