import numpy as np
import matplotlib.pyplot as mp

import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def dcurve_dt(t):
    dx = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * np.cos(2.0 * t) - \
        (3.0 / 4.0) * np.sin(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    dy = np.cos(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0) + \
        np.sin(t - np.pi / 4.0) * np.cos(4.0 * t) * 2.0

    return np.array([dx, dy])


def plot_skeleton_row(matrix, qbx, filename):
    node = matrix
    m = 0
    while True:
        for i in range(node.nranges):
            mp.plot(qbx.x[0, node.rindex[i]],
                    qbx.x[1, node.rindex[i]], "ko", alpha=0.2)
            mp.plot(qbx.x[0, node.compressed_rindex[i]],
                    qbx.x[1, node.compressed_rindex[i]], "o")

        mp.xlim([-2.0, 2.0])
        mp.ylim([-2.0, 2.0])
        mp.savefig("{}_{:02}.png".format(filename, m),
                   bbox_inches="tight", dpi=300)
        mp.clf()

        m = m + 1
        node = node.parent
        if node is None:
            break


def plot_skeleton_col(matrix, qbx, filename):
    node = matrix
    m = 0
    while True:
        for i in range(node.nranges):
            mp.plot(qbx.x[0, node.cindex[i]],
                    qbx.x[1, node.cindex[i]], "ko", alpha=0.2)
            mp.plot(qbx.x[0, node.compressed_cindex[i]],
                    qbx.x[1, node.compressed_cindex[i]], "o")

        mp.xlim([-2.0, 2.0])
        mp.ylim([-2.0, 2.0])
        mp.savefig("{}_{:02}.png".format(filename, m),
                   bbox_inches="tight", dpi=300)
        mp.clf()

        m = m + 1
        node = node.parent
        if node is None:
            break


def plot_skeleton_matrix(matrix, qbx, filename):
    node = matrix
    m = 0
    while True:
        E = np.zeros(D.shape)
        for i in range(node.nranges):
            for j in range(node.nranges):
                block = np.ix_(node.compressed_rindex[i],
                               node.compressed_cindex[j])
                E[block] = 1.0
        mp.matshow(E)
        mp.colorbar()

        mp.savefig("{}_{:02}.png".format(filename, m),
                   bbox_inches="tight", dpi=300)
        mp.clf()

        m = m + 1
        node = node.parent
        if node is None:
            break


def plot_skeleton_decomp(matrix, qbx, filename):
    node = matrix
    m = 0
    while True:
        L = np.zeros((node.shape[0], node.cshape[1]))
        for i in range(node.nranges):
            rindex = np.arange(node.rranges[i], node.rranges[i + 1])
            cindex = np.arange(node.compressed_cranges[i],
                               node.compressed_cranges[i + 1])

            L[np.ix_(rindex, cindex)] = node.L[i, i]

        mp.imshow(L)
        mp.colorbar()
        mp.savefig("{}_l_{:02}.png".format(filename, m),
                   bbox_inches="tight", dpi=300)
        mp.clf()

        L = np.zeros((node.cshape[0], node.shape[1]))
        for i in range(node.nranges):
            rindex = np.arange(node.compressed_rranges[i],
                               node.compressed_rranges[i + 1])
            cindex = np.arange(node.cranges[i], node.cranges[i + 1])

            L[np.ix_(rindex, cindex)] = node.R[i, i]

        mp.imshow(L)
        mp.colorbar()
        mp.savefig("{}_r_{:02}.png".format(filename, m), dpi=300)
        mp.clf()

        L = np.zeros((node.shape[0], node.shape[1]))
        for i in range(node.nranges):
            rindex = np.arange(node.rranges[i], node.rranges[i + 1])
            cindex = np.arange(node.cranges[i], node.cranges[i + 1])

            L[np.ix_(rindex, cindex)] = node.D[i, i]

        mp.imshow(L)
        mp.colorbar()
        mp.savefig("{}_d_{:02}_diag.png".format(filename, m), dpi=300)
        mp.clf()

        L = np.zeros((node.cshape[0], node.cshape[1]))
        mask = np.arange(0, node.nranges)
        for i in range(node.nranges):
            rindex = np.arange(node.compressed_rranges[i],
                               node.compressed_rranges[i + 1])
            cindex = np.arange(node.compressed_cranges[i],
                               node.compressed_cranges[i + 1])
            L[np.ix_(rindex, cindex)] = node.M[i, i]

            for j in mask[mask != i]:
                rindex = np.arange(node.compressed_rranges[i],
                                   node.compressed_rranges[i + 1])
                cindex = np.arange(node.compressed_cranges[j],
                                   node.compressed_cranges[j + 1])
                block = np.ix_(node.compressed_rindex[i],
                               node.compressed_cindex[j])

                L[np.ix_(rindex, cindex)] = node.A[block]

        mp.imshow(L)
        mp.colorbar()
        mp.savefig("{}_s_{:02}.png".format(filename, m), dpi=300)
        mp.clf()

        m = m + 1
        node = node.parent
        if node is None:
            break


if __name__ == "__main__":
    # order of QBX expansion
    p = 3
    # nodes in Gauss quadrature
    q = 7
    # number of panels
    npanels = 30
    # number of proxy points
    nproxies = 64
    # radius ratio
    ratio = 1.25
    # maximum level
    maxlevel = int(np.log2(q + 1))

    # define geometry
    geometry = cs.ParametricGeometry(curve, dcurve_dt)
    # define solution representation
    K = cs.DoubleLayerKernel(cs.GreenLaplace())
    K = cs.ExpandedKernel(K, order=p)
    # define quadrature
    qbx = cs.QBX(geometry, npanels, order=q, adapt=True)
    # define integral operator
    D = qbx.boundary_operator(K, limit=cs.QBXExpansion.Boundary)
    D = 0.5 * np.eye(D.shape[0]) + D

    matrix = cs.InterpolativeMatrix(D, maxlevel=maxlevel)
    plot_skeleton_row(matrix, qbx, "matrix_id_row")
    plot_skeleton_col(matrix, qbx, "matrix_id_col")
    plot_skeleton_matrix(matrix, qbx, "matrix_id_nnz")
    plot_skeleton_decomp(matrix, qbx, "matrix_id_blk")

    matrix = cs.SkeletonMatrix(D, maxlevel=maxlevel,
                               ratio=ratio, nproxies=nproxies,
                               sources=qbx, kernel=K)
    plot_skeleton_row(matrix, qbx, "matrix_sk_row")
    plot_skeleton_col(matrix, qbx, "matrix_sk_col")
    plot_skeleton_matrix(matrix, qbx, "matrix_sk_nnz")
    plot_skeleton_decomp(matrix, qbx, "matrix_sk_blk")
