import numpy as np
import numpy.linalg as nl

import cs598apk as cs


def curve(t):
    x = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    y = np.sin(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0)

    return np.array([x, y])


def dcurve_dt(t):
    dx = (3.0 / 4.0) * np.cos(t - np.pi / 4.0) * np.cos(2.0 * t) - \
        (3.0 / 4.0) * np.sin(t - np.pi / 4.0) * (1.0 + np.sin(2.0 * t) / 2.0)
    dy = np.cos(t - np.pi / 4.0) * (1.0 + np.sin(4.0 * t) / 2.0) + \
        np.sin(t - np.pi / 4.0) * np.cos(4.0 * t) * 2.0

    return np.array([dx, dy])


def test_constant_density(npanels, limit, sigma_const):
    # number of quadrature nodes
    default_q = 15
    # expansion order
    default_p = 3

    for p in [3, 5, 7]:
        # define geometry
        geometry = cs.ParametricGeometry(curve, dcurve_dt)
        # define solution representation
        K = cs.DoubleLayerKernel(cs.GreenLaplace())
        K = cs.ExpandedKernel(K, order=p)
        # define quadrature
        quad = cs.QBX(geometry, npanels, order=default_q)
        # compute operator
        D = quad.boundary_operator(K, limit=limit)

        # apply to constant density
        sigma = np.full(D.shape[1], sigma_const)
        if limit == cs.QBXExpansion.Interior:
            error = nl.norm(D @ sigma - sigma_const, float("inf"))
        elif limit == cs.QBXExpansion.Exterior:
            error = nl.norm(D @ sigma - 0.0, float("inf"))
        else:
            error = nl.norm(D @ sigma - 0.5 * sigma_const, float("inf"))

        print(error)

    for q in [7, 11, 15]:
        # define geometry
        geometry = cs.ParametricGeometry(curve, dcurve_dt)
        # define solution representation
        K = cs.DoubleLayerKernel(cs.GreenLaplace())
        K = cs.ExpandedKernel(K, order=default_p)
        # define quadrature
        quad = cs.QBX(geometry, npanels, order=q)
        # construct operator
        D = quad.boundary_operator(K, limit=limit)

        # apply to constant density
        sigma = np.full(D.shape[1], sigma_const)
        if limit == cs.QBXExpansion.Interior:
            error = nl.norm(D @ sigma - sigma_const, float("inf"))
        elif limit == cs.QBXExpansion.Exterior:
            error = nl.norm(D @ sigma - 0.0, float("inf"))
        else:
            error = nl.norm(D @ sigma - 0.5 * sigma_const, float("inf"))

        print(error)


if __name__ == "__main__":
    # initial number of panels
    npanels = 100

    limit = cs.QBXExpansion.Interior
    test_constant_density(npanels, limit, np.pi)
    limit = cs.QBXExpansion.Exterior
    test_constant_density(npanels, limit, np.pi)
    limit = cs.QBXExpansion.Boundary
    test_constant_density(npanels, limit, np.pi)
